#!/usr/bin/env python
# -*- coding: utf-8 -*-
r"""
@DATE: 2024/6/14 22:54
@File: __init__.py.py
@IDE: pycharm
@Description:
    swankit —— SwanLab Toolkit
    为 SwanLab 提供的工具包，包含了一些常用的工具函数、类、模块等
    为了语意方便，应该通过swankit的不同模块调用各个功能函数
"""

__all__ = ["env", "callback", "log"]
